<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Factory\IntegracaoHcmfactory;
use Teknisa\Libs\Util\Environment;

class IntegracaoHcmService {
    
    protected $hcmFactory;
    protected $environment;
    protected $entityManager;
    protected $connection;
    
    public function __construct(Environment $environment, EntityManager $entityManager, IntegracaoHcmFactory $IntegracaoHcmFactory){
        $this->environment   = $environment;
        $this->entityManager = $entityManager;
        $this->connection    = $entityManager->getConnection();
        $this->hcmFactory    = $IntegracaoHcmFactory;
    }
    
    /**
     * Efetua as configurações necessárias para o usuário do CRM
     * salvar a proposta
     * @params array $rows dados da proposta recebidos do CRM
    **/
    // public function integracaoHcm($rows){
    //     try{
    //         $nrorg = $rows["requisitante"]["nrorg"];
    //         $entityManagerConnection = $this->connectionCustom->updateVpdConnection($nrorg);
            
    //         if ($entityManagerConnection !== false) {
    //             $this->entityManager = $entityManagerConnection;
    //         }
            
    //         $this->connection = $this->entityManager->getConnection();
    //         $this->newCode = new NewCode($this->connection, "NOVO_CODIGO_SC");
    //         $this->integracaoHcmfactory = new IntegracaoHcmfactory($this->entityManager);
            
    //         $retorno = $this->buscaLancamentosDeCalculo($rows, $nrorg);
            
    //         return $retorno;
    //     } catch(\Exception $e) {
    //         throw $e;
    //     }
    // }
    
    /**
     * Salva a proposta com os dados recebidos do crm
     * @params array $rows dados da proposta recebidos do CRM
    **/
    // public function buscaLancamentosDeCalculo($rows, $nrorg){
    //     try{
    //         $this->connection->beginTransaction();
            
    //         $dtmescompetenc = $rows[""]["DTMESCOMPETENC"];
    //         $nrtipomoviment = $rows[""]["NRTIPOMOVIMENT"];
    //         $nrcpfpessoa = $rows[""]["NRCPFPESSOA"];
            
    //         $informacoesHoraExtra = $this->retornaLancamentosDeCalculo($nrorg, $dtmescompetenc, $nrtipomoviment, $nrcpfpessoa);

    //         $this->connection->commit();
    //         return $informacoesHoraExtra;
            
    //     } catch(\Exception $e) {
    //         $this->connection->rollback();
    //         throw $e;
    //     }    
    // }
    
    /**
     * Retorna os dados da proposta presentes no modelo
     * @params $nrorg nr organização
     * @params $nrSeqOrcamentoModelo nr do modelo
    **/
    // public function retornaLancamentosDeCalculo($nrorg, $dtmescompetenc, $nrtipomoviment, $nrcpfpessoa){
    //     $paramsHoraExtra = array(
    //         "DTMESCOMPETENC" => $dtmescompetenc,
    //         "NRTIPOMOVIMENT" => $nrtipomoviment,
    //         "NRCPFPESSOA" => $nrcpfpessoa,
    //         "NRORG"             => $nrorg
    //     );
        
    //     $dataSet = $this->integracaoHcmfactory->retornaLancamentosDeCalculo($paramsHoraExtra);
    //     return $dataSet;
    // }

    public function integracaoHcm($params)
    {
        $token = '';
        $paramGeral = $this->hcmFactory->getParamsGeral();
        $access = $this->checkAccess($paramGeral[0]['DSURLHCM'], $paramGeral[0]['DSTOKENHCM'], $paramGeral[0]['DSSECRETHCM']);
       
        if($access == 'Denied'){
            $this->connection->beginTransaction();
            $token = $this->requestToken($paramGeral[0]['DSURLHCM'], $paramGeral[0]['DSIDHCM'], $paramGeral[0]['DSSECRETHCM']);
            $this->hcmFactory->updateToken($token);
            $this->connection->commit();
        } else {
            $token = $paramGeral[0]['DSTOKENHCM'];
        }

        return $this->setEstrutura($params, $paramGeral[0]['DSURLHCM'], $token, $paramGeral[0]['DSSECRETHCM']);
    }

    public function checkAccess($uri, $token, $secret)
    {
        $uri_produ = $uri.'/checkAccess';
        $clientData = array(
            "TOKEN" => $token,
            "CLIENT_SECRET" => $secret,
        );
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($clientData));
        $result = $this->connectionWebService($header, $uri_produ, $clientData);
        if ($result['messages'][0]['message'] == "Authorized") {
            return "Authorized";
        } else {
            return "Denied";
        }
    }

    public function requestToken ($uri, $id, $secret) 
    {
        $uri_produ = $uri.'/requestToken';
        $clientData = array(
            "CLIENT_ID" => $id,
            "CLIENT_SECRET"=> $secret,
        );
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($clientData));
        $result = $this->connectionWebService($header, $uri_produ, $clientData);
        return $result['dataset']['token'][0];
    }


    /**
     * Função responsável por criar estrutura
     *
     * @param Request\Row $request  No request são esperados os campos:
     * Informações obrigatórias:
     * - NRTIPOESTRUTURA : Número do tipo de Estrutura conforme descrição abaixo:
     *      1- EMPRESA
     *      2- FILIAL
     *      3- DEPARTAMENTO
     *      4- SETOR
     *      5- CENTRO CUSTO
     *      6- EQUIPE
     *      7- TOMADOR DE SERVICOS
     *      8- REGIÃO
     *      9- PROJETO
     *      10- SINDICATO
     *      11- OBRA
     *      12- PRODUTO
     *      13- PLANO DE SAÚDE
     *      14- ESCOLA
     *      25- LABORATÓRIO
     *      26- FORNECEDOR
     * - DTINIVIGENCIA : Data de início de vigência da respectiva estrutura (caso não seja informada, ficará com a data 01/01/2000).
     * - NMFANTASIA: Nome fantasia do cliente.
     * - NMRAZSOCESTRUT: Razão social do cliente.
     * - CDCNPJESTRUT: Nº do CNPJ do Cliente.
     * - NRCEPENDERECO: CEP do endereço da estrutura.
     * - NMPAIS: Nome do país do endereço da estrutura.
     * - SGESTADO: Sigla do estado do endereço da estrutura.
     * - NMMUNICIPIO: Nome do município da estrutura.
     * - NMBAIRROENDERECO: Nome do bairro do endereço da estrutura.
     * - CDLOGRADOURO: Descrição do tipo de logradouro do endereço da estrutura (rua, avenida,…).
     * - DSENDERECO: Descrição do endereço da estrutura.
     * - NRIMOVELENDERECO: número da estrutura do endereço da estrutura.
     * - DSCOMPLEENDERECO: Descrição do complemento do endereço da estrutura.
     * - DSREFERENCIAENDE: Descrição da referência do endereço da estrutura.
     *
     *  Informações não obrigatórias:
     * - IDTIPOEMPR: Tipo de empresa conforme descrição abaixo:
     *      1 – Microempresa
     *      2 – Empresa de Pequeno Porte
     *      3 – Empresa/Órgão
     *      4 – Microempreendedor Individual
     *      5 – Empregador Doméstico
     *      6 – Empregador Rural
     * - IDOPTSIMPLES: Indica se a empresa é optante pelo simples (opções “S” ou “N”).
     * - IDPARTICIPAT: Indica se a empresa participa do “Programa de Alimentação do Trabalhador”.
     * - NRTIPOTRIBUT: tipo de tributação da empresa conforme opções abaixo:
     *      1 – Lucro Real
     *      2 – Lucro Presumido
     *      3 – Simples Nacional
     * - NRCLASSTRIBUT: código da classificação tributária (conforme tabela CLASSTRIBUT).
     * - VRALIQDESON: valor percentual da alíquota de desoneração.
     * - NRCALENDARIO: número do calendário da estrutura.
     * - CDCNAE: Código do CNAE da empresa.
     * - CDFPAS: Código do FPAS.
     * - CDTERCEIRO: código de terceiros.
     * - VRFAP: Valor do Fator Acidentário de Prevenção (FAP) da empresa.
     * - CDCEIESTRUT: CEI da empresa.
     * - NRCAGED: Código CAGED.
     * - CDEMPRCAIXA: Código de FGTS na Caixa Econômica Federal.
     * - CDFEDERALESTRUT: Código do registro federal (exemplo: código ANS do plano de saúde).
     * - NRPROPRIEMPR: CPF do proprietário.
     * - NRPARCRESPINF: CPF do responsável informal.
     * - NRPARCENTISIN: CPF do responsável legal.
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [estrutura] = Array (
     *          true
     *      )
     *  )
     * )
     */
    
     public function setEstrutura($params, $uri, $token, $secret)
    {
        $nrEstrutura = 7;
        $contrato = $this->hcmFactory->getAllContracts($params['NRSEQCONTRATO']);
        $uri_produ = $uri."/setEstrutura";
        $header = array(
            "OAuth-Token: $token",
            "OAuth-SecretId: $secret"
        );
        $header[] = 'Content-Type: application/json';
        $data = array(
            'requestType' => 'Row',
                'row' => array(
                    "NRTIPOESTRUTURA"   => $nrEstrutura,
                    "DTINIVIGENCIA"     => $contrato[0]['DTINICIOCONTRATO'],
                    "NMFANTASIA"        => $contrato[0]['NMPRINCIPALPARC'],
                    "NMRAZSOCESTRUT"    => $contrato[0]['NMPRINCIPALPARC'],
                    "CDCNPJESTRUT"      => $contrato[0]['NRINSCRICAOPARC'] ,
                    "NRCEPENDERECO"     => $contrato[0]['NRCEPENDERECO'],
                    "NMPAIS"            => $contrato[0]['NMPAIS'],
                    "SGESTADO"          => $contrato[0]['SGESTADO'],
                    "NMMUNICIPIO"       => $contrato[0]['NMMUNICIPIO'],
                    "NMBAIRROENDERECO"  => $contrato[0]['NMBAIRROENDERECO'],
                    "CDLOGRADOURO"      => ($contrato[0]['CDLOGRADOURO'] == NULL) ? 0 : $contrato[0]['CDLOGRADOURO'],
                    "DSENDERECO"        => ($contrato[0]['DSCOMPLEENDERECO'] == null) ? 'Não há' : $contrato[0]['DSCOMPLEENDERECO'],
                    "NRIMOVELENDERECO"  => ($contrato[0]['NRIMOVELENDERECO'] == NULL) ? 0 : $contrato[0]['NRIMOVELENDERECO'],
                    "DSCOMPLEENDERECO"  => ($contrato[0]['DSCOMPLEENDERECO'] == NULL) ? 'Não há' : $contrato[0]['DSCOMPLEENDERECO'],
                    "DSREFERENCIAENDE"  => ($contrato[0]['DSREFERENCIAENDE'] == NULL) ? 'Não há' : $contrato[0]['DSREFERENCIAENDE'],
                    "IDOPTSIMPLES"      => 'N',
            ));

        $estrutura = $this->connectionWebService($header, $uri_produ, $data);

        if((isset($estrutura['dataset']['setEstrutura'][0]) && $estrutura['dataset']['setEstrutura'][0]) 
        || (isset($estrutura['messages'][0]['message']) && $estrutura['messages'][0]['message'] == 'Já existe estrutura para o mesmo tipo de estrutura e CNPJ')) {
          return $this->setQuadroVagas($uri, $token, $secret, $nrEstrutura, $contrato[0]['NRINSCRICAOPARC'], $params['NRSEQCONTRATO'], $contrato[0]['DTFIMCONTRATO']);
        } else {
            return 'NOK';
        }
    }

    /**
     * Função responsável por criar um quadro de vagas
     *
     * @param Request\Row $request  No request são esperados os campos:
     * Informações obrigatórias:
     * - CDCNPJESTRUT: CNPJ da estrutura
     * - NRTIPOESTRUTURA : Número do tipo de Estrutura conforme descrição abaixo:
     *      1- EMPRESA
     *      2- FILIAL
     *      3- DEPARTAMENTO
     *      4- SETOR
     *      5- CENTRO CUSTO
     *      6- EQUIPE
     *      7- TOMADOR DE SERVICOS
     *      8- REGIÃO
     *      9- PROJETO
     *      10- SINDICATO
     *      11- OBRA
     *      12- PRODUTO
     *      13- PLANO DE SAÚDE
     *      14- ESCOLA
     *      25- LABORATÓRIO
     *      26- FORNECEDOR
     * - CDOCUPACAO: Código de integração da ocupação
     * - QTDLIMITE: Quantidade limite do quadro de vagas
     * - DTINIVALIDADE: Data de início de validade do quadro de vagas no formato DD/MM/YYYY
     * Informações não obrigatórias:
     * - DTFIMVALIDADE: Data fim de validade do quadro de vagas no formato DD/MM/YYYY
     * - CDPROPOSTA: Código da proposta
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [quadro] = Array (
     *          true
     *      )
     *  )
     * )
     */
    public function setQuadroVagas($uri, $token, $secret, $estrutura, $cnpj, $params, $dtfinal)
    {
        $itensContrato = $this->hcmFactory->getItensContrato($params);
        $uri_produ = $uri."/setQuadroVagas";
        $header = array(
            "OAuth-Token: $token",
            "OAuth-SecretId: $secret"
        );
        $header[] = 'Content-Type: application/json';
        foreach ($itensContrato as $value) {
            $data = array(
                'requestType' => 'Row',
                'row' => array(
                    "CDCNPJESTRUT"      => $cnpj,
                    "NRTIPOESTRUTURA"   => $estrutura,
                    "CDOCUPACAO"        => $value['CDSERVORCA'],
                    "QTDLIMITE"         => $value['QTITEMCONTRATO'],
                    "DTINIVALIDADE"     => $dtfinal,
                    "CDPROPOSTA"        => $value['NRITEMOCONTRATO']
                )
            );
            var_dump(count($itensContrato), $data);
            die;
            $quadro = $this->connectionWebService($header, $uri_produ, $data);
        }
    }

    public function connectionWebService($header, $uri_produ, $data) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $uri_produ);
        curl_setopt($ch, CURLOPT_POST, count($data));
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $result = curl_exec($ch);
        curl_close($ch);
        return json_decode($result, true);
    }
   
   
}
